@@ -96,7 +96,7 @@ def from_array(
     # parse data
     fd = FTYPES[ftype]
     if check_ftype and not fd.isvalid(data):
-        raise ValueError(f'The flow direction type "{ftype}" is not recognized.')
+        raise ValueError(f'The flow direction data with type "{ftype}" is invalid.')
     if mask is not None:
         if mask.shape != data.shape:
             raise ValueError(f'"mask" shape does not match with data shape')
@@ -669,8 +669,7 @@ class FlwdirRaster(object):
         return basids.reshape(self.shape)
 
     def basin_bounds(self, basins=None, **kwargs):
-        """Returns a table with the basin boundaries. At index 0 the total bounding
-        box is given.
+        """Returns a the basin boundaries.
 
         Additional key-word arguments are passed to the basins method which is used to
         create a basins map if none is provided.
@@ -682,22 +681,49 @@ class FlwdirRaster(object):
 
         Returns
         -------
-        pandas.DataFrame
-            bounds of basins
+        lbs : 1D array of int
+            labels of basins
+        bboxs : 2D array of float
+            bounding boxes of basins, the columns represent [xmin, ymin, xmax, ymax]
+        total_bbox : 1D array of float
+            the total bounding box of all basins [xmin, ymin, xmax, ymax]
         """
         if basins is None:
             basins = self.basins(**kwargs)
         elif basins.size != self.size:
             raise ValueError('"basins" size does not match with FlwdirRaster size')
-        return regions.region_bounds(basins, transform=self.transform)
+        lbs, bboxs, total_bbox = regions.region_bounds(basins, transform=self.transform)
+        return lbs, bboxs, total_bbox
 
-    def pfafstetter(self, idx0, depth=1, uparea=None, upa_min=0.0):
-        """Returns the pfafstetter coding for a single basin.
+    def subbasins(self, strord=None, min_sto=0):
+        """Returns a subbasin map with unique random IDs.
+
+        Parameters
+        ----------
+        strord : 1D-array of uint8
+            stream order
+        min_sto : int, optional
+            minimum stream order of subbasins, by default the stream order is set to
+            two under the global maxmium stream order.
+
+        Returns
+        -------
+        basins : 1D-arrays of uint32
+            map with unique IDs for stream_order>=min_sto subbasins
+        """
+        subbas = basins.subbasins(
+            idxs_ds=self.idxs_ds,
+            seq=self.idxs_seq,
+            strord=self._check_data(strord, "strord"),
+            min_sto=min_sto,
+        )
+        return subbas.reshape(self.shape)
+
+    def pfafstetter(self, depth=1, uparea=None, upa_min=0.0):
+        """Returns the pfafstetter subbasins.
 
         Parameters
         ----------
-        idx0 : int
-            index of outlet cell
         depth : int, optional
             Number of pfafsterrer layers, by default 1.
         uparea : 2D array of float, optional
@@ -711,7 +737,7 @@ class FlwdirRaster(object):
             subbasin map with pfafstetter coding
         """
         pfaf = basins.pfafstetter(
-            idx0=idx0,
+            idxs_pit=self.idxs_pit,
             idxs_ds=self.idxs_ds,
             seq=self.idxs_seq,
             uparea=self._check_data(uparea, "uparea"),
@@ -841,42 +867,77 @@ class FlwdirRaster(object):
         )
         return stream_dist.reshape(self.shape)
 
-    def vectorize(self, mask=None, xs=None, ys=None, crs=None):
-        """Returns a GeoDataFrame with river segments. Segments are LineStrings
-        connecting cell from down to upstream.
+    # TODO remove in v0.5
+    def vectorize(self, **kwargs):
+        """NOTE: this method will be deprecated from v0.5"""
+        warnings.warn(
+            "vectorize will be deprecated in v0.5. Use features instead.",
+            PendingDeprecationWarning,
+        )
+        import geopandas as gp
 
-        The coordinates of the nodes are based on the cell center as calculated
-        using the affine transform, unless maps with subgrid x and y
+        df = gp.GeoDataFrame.from_features(self.features(kind="flwdir", **kwargs))
+        return df.set_index("idx")
+
+    def features(
+        self, kind="streams", mask=None, xs=None, ys=None, min_sto=1, **kwargs
+    ):
+        """Returns a geo-features of streams of same order or local flow direction.
+
+        The coordinates are based on the cell center as calculated
+        using the affine transform, unless maps with (subgrid) x and y
         coordinates are provided.
 
         Parameters
         ----------
-        min_order : int
+        kind : {streams, flwdir}
+            Kind of LineString features: either streams of local flow directions.
+        mask : ndarray of bool
+            Maks of valid cells.
+        min_sto : int
             Minimum Strahler Order recognized as river, by the default 1.
+            Only in combination with kind = 'streams'
         xs, ys : ndarray of float
             Raster with cell x, y coordinates, by default None and inferred from cell
             center.
-        crs : str, optional
-            Coordinate reference system used for the returned GeoDataFrame
+        kwargs : extra sample maps key-word arguments
+            optional maps to sample from
+            e.g.: strord=flw.stream_order()
 
         Returns
         -------
-        geopandas.GeoDataFrame
-            flow direction vector
+        feats : list of dict
+            Geofeatures, to be parsed by e.g. geopandas.GeoDataFrame.from_features
         """
+        if "stream" in kind:
+            if "strord" not in kwargs:
+                kwargs.update(strord=self.stream_order())
+            idxs = streams.streams(
+                idxs_ds=self.idxs_ds,
+                seq=self.idxs_seq,
+                strord=self._check_data(kwargs["strord"], "strord"),
+                mask=self._check_data(mask, "mask", optional=True),
+                min_sto=min_sto,
+            )
+        elif kind == "flwdir":
+            idxs = core.flwdir_tuples(
+                self.idxs_ds,
+                mask=self._check_data(mask, "mask", optional=True),
+                mv=self._mv,
+            )
+        else:
+            ValueError('Kind should be either "streams" or "flwdir"')
         # get geoms and make geopandas dataframe
         if xs is None or ys is None:
-            idxs = np.arange(self.size, dtype=np.intp)
-            xs, ys = gis.idxs_to_coords(idxs, self.transform, self.shape)
-        df = gis.vectorize(
-            idxs_ds=self.idxs_ds,
+            idxs0 = np.arange(self.size, dtype=np.intp)
+            xs, ys = gis.idxs_to_coords(idxs0, self.transform, self.shape)
+        feats = gis.features(
+            streams=idxs,
             xs=self._check_data(xs, "xs"),
             ys=self._check_data(ys, "ys"),
-            mask=self._check_data(mask, "mask", optional=True),
-            crs=crs,
-            mv=self._mv,
+            **kwargs,
         )
-        return df
+        return feats
 
     ### UPSCALE FLOW DIRECTIONOS ###
 
@@ -1376,6 +1437,8 @@ class FlwdirRaster(object):
             data = self.upstream_area(**kwargs)
         elif data is None and name == "basins":
             data = self.basins(**kwargs)
+        elif data is None and name == "strord":
+            data = self.stream_order(**kwargs)
         elif not np.atleast_1d(data).size == self.size:
             raise ValueError(f'"{name}" size does not match with FlwdirRaster size')
         return np.atleast_1d(data).ravel()
